<?php

namespace Repetito\Admin;

// If this file is called directly, abort.
if ( ! defined( 'Repetito\NAME' ) ) {
	exit;
}

use Repetito\Traits\Singleton;
use const Repetito\FILE;
use const Repetito\NAME;
use const Repetito\VERSION;

final class Menu {

	use Singleton;


	/**
	 * It registers menu page for admin.
	 * @return void
	 */
	public function register() {
		add_menu_page(
			__( 'Repetito', 'repetito' ),
			__( 'Repetito', 'repetito' ),
			'manage_options',
			'repetito',
			array( $this, 'render' ),
			'dashicons-plugins-checked',
			6
		);
	}

	/**
	 * It will display dashboard template. Also remove admin footer text.
	 * @return void
	 */
	public function render() {
		add_filter( 'admin_footer_text', '__return_empty_string', 11 );
		add_filter( 'update_footer', '__return_empty_string', 11 );
		echo wp_kses_post( '<div id="' . NAME . '"></div>' );
	}

	/**
	 * It will add module attribute to script tag.
	 *
	 * @param string $tag of script.
	 * @param string $id of script.
	 *
	 * @return string
	 */
	public function add_module( string $tag, string $id ): string {
		if ( NAME === $id ) {
			$tag = str_replace( '<script ', '<script type="module" ', $tag );
		}

		return $tag;
	}

	/**
	 * @return void
	 */
	public function scripts() {
			wp_enqueue_style( NAME, plugins_url( 'assets/admin/dist/style.css', FILE ), array(), VERSION );
			wp_enqueue_script( NAME, plugins_url( 'assets/admin/dist/index.js', FILE ), array(), VERSION, true );
		if ( ! did_action( 'wp_enqueue_media' ) ) {
			wp_enqueue_media();
		}
		add_action( 'wp_enqueue_scripts', 'wp_enqueue_media', 999 );

		wp_localize_script(
			NAME,
			'repetito',
			array(
				'nonce'        => wp_create_nonce( 'wp_rest' ),
				'api_endpoint' => get_rest_url( null, 'repetito/v1/' ),
			)
		);
	}
}
