<?php

namespace Repetito\Api;

// If this file is called directly, abort.
if ( ! defined( 'Repetito\NAME' ) ) {
	exit;
}

use WP_Error;
use WP_REST_Server;
use WP_REST_Request;
use WP_REST_Response;
use Repetito\Traits\Api;
use Repetito\Traits\Singleton;

final class Fees {
	use Api;
	use Singleton;

	/**
	 * Register the routes for serving data from custom table
	 */
	public function __construct() {
		register_rest_route(
			$this->namespace,
			'/fees',
			array(
				array(
					'methods'             => WP_REST_Server::READABLE,
					'callback'            => array( $this, 'fees' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => array(),
				),
			)
		);
		register_rest_route(
			$this->namespace,
			'/fees',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'fees' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => array(),
				),
			)
		);
	}

	/**
	 * Get data from custom table
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function fees( WP_REST_Request $request ) {
		$option_key = 'repetito-fees';
		$options    = get_option( $option_key, array() );
		if ( $request->get_method() === WP_REST_Server::CREATABLE ) {
			$update            = array();
			$update['price']   = $request->get_param( 'price' ) != '' ? (float) wp_kses_post( $request->get_param( 'price' ) ) : '';
			$update['tax']     = $request->get_param( 'tax' ) != '' ? wp_kses_post( $request->get_param( 'tax' ) ) : '';
			$update['weight']  = $request->get_param( 'weight' ) != '' ? wp_kses_post( $request->get_param( 'weight' ) ) : '';
			$update['country'] = $request->get_param( 'country' ) != '' ? wp_kses_post( $request->get_param( 'country' ) ) : '';
			$options           = array_merge( $options, $update );
			update_option( $option_key, $options );
		}

		$options['countries'] = $this->getAvailableCountries();
		$options['taxes']     = $this->getAvailableTaxClasses();

		return rest_ensure_response(
			array(
				'data'    => $options,
				'message' => __( 'Príplatok aktualizován', 'repetito' ),
			)
		);
	}

	/**
	 * Get all possible shipping countries
	 *
	 * @return Array
	 */
	private function getAvailableCountries() {
		$wc_countries = new \WC_Countries();
		return array_map(
			function ( $c_val, $c_key ) {
				return array(
					'value' => $c_key,
					'text'  => $c_val,
				);
			},
			array_values( $wc_countries->get_countries() ),
			array_keys( $wc_countries->get_countries() )
		);
	}

	/**
	 * Get all tax classes
	 *
	 * @return Array
	 */
	private function getAvailableTaxClasses() {
		$tax_classes                   = \WC_Tax::get_tax_classes();
		$tax_class_options             = array();
		$tax_class_options['standard'] = __( 'Standard', 'woocommerce' );

		if ( ! empty( $tax_classes ) ) {
			foreach ( $tax_classes as $class ) {
				$tax_class_options[ sanitize_title( $class ) ] = $class;
			}
		}

		return array_map(
			function ( $c_val, $c_key ) {
				return array(
					'value' => $c_key,
					'text'  => $c_val,
				);
			},
			array_values( $tax_class_options ),
			array_keys( $tax_class_options )
		);
	}
}
