<?php

namespace Repetito\Api;

// If this file is called directly, abort.
if ( ! defined( 'Repetito\NAME' ) ) {
	exit;
}

use WP_Error;
use WP_REST_Server;
use WP_REST_Request;
use WP_REST_Response;
use Repetito\Traits\Api;
use Repetito\Traits\Singleton;
use Repetito\Web\Frontend;

final class Packages {
	use Api;
	use Singleton;

	/**
	 * Register the routes for serving data from custom table
	 */
	public function __construct() {
		register_rest_route(
			$this->namespace,
			'/packages',
			array(
				array(
					'methods'             => WP_REST_Server::READABLE,
					'callback'            => array( $this, 'packages' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => array(),
				),
			)
		);
		register_rest_route(
			$this->namespace,
			'/packages',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'updatePackage' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => array(),
				),
			)
		);
	}

	public function updatePackage( WP_REST_Request $request ) {
		$package_name = $request->get_param( 'name' );
		update_option( 'repetito-packages_' . $package_name, (bool) $request->get_param( 'active' ) );

		return rest_ensure_response(
			array(
				'data'    => (bool) $request->get_param( 'active' ),
				'message' => __( 'Nastavenie aktualizováno', 'repetito' ),
			)
		);
	}

	public function getPackagesForApi() {
		$packagesInstance = Frontend::instance();
		$packages         = $packagesInstance::PACKAGES;
		return array_map(
			function ( $p ) {
				$p['active'] = (bool) get_option( 'repetito-packages_' . $p['name'], false );
				return $p;
			},
			$packages
		);
	}

	/**
	 * Get data from custom table
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function packages( WP_REST_Request $request ) {

		return rest_ensure_response(
			array(
				'data'    => $this->getPackagesForApi(),
				'message' => __( 'Nastavenie aktualizováno', 'repetito' ),
			)
		);
	}
}
