<?php

namespace Repetito\Api;

// If this file is called directly, abort.
if ( ! defined( 'Repetito\NAME' ) ) {
	exit;
}

use WP_Error;
use WP_REST_Server;
use WP_REST_Request;
use WP_REST_Response;
use Repetito\Traits\Api;
use Repetito\Traits\Singleton;

final class Settings {
	use Api;
	use Singleton;

	/**
	 * Register the routes for serving data from custom table
	 */
	public function __construct() {
		register_rest_route(
			$this->namespace,
			'/settings',
			array(
				array(
					'methods'             => WP_REST_Server::READABLE,
					'callback'            => array( $this, 'settings' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => array(),
				),
			)
		);

		register_rest_route(
			$this->namespace,
			'/settings',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'settings' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => array(),
				),
			)
		);
	}

	/**
	 * Get data from custom table
	 *
	 * @param WP_REST_Request $request Full data about the request.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function settings( WP_REST_Request $request ) {
		$option_key = 'repetito-settings';
		$options    = get_option( $option_key, array() );

		if ( $request->get_method() === WP_REST_Server::CREATABLE ) {
			$update                  = array();
			$update['serviceActive'] = $request->get_param( 'serviceActive' ) === 'true';
			$update['showImage']     = $request->get_param( 'showImage' ) === 'true';
			$options                 = array_merge( $options, $update );
			update_option( $option_key, $options );
		}

		return rest_ensure_response(
			array(
				'data'    => $options,
				'message' => __( 'Nastavenie aktualizováno', 'repetito' ),
			)
		);
	}
}
