<?php

namespace Repetito\Web;

// If this file is called directly, abort.
if ( ! defined( 'Repetito\NAME' ) ) {
	exit;
}

require_once __DIR__ . '/BoxPacker.php';
require_once __DIR__ . '/TestBox.php';


use Repetito\Traits\Singleton;
use const Repetito\NAME;
use Repetito\BoxPacker;

class Frontend {


	use Singleton;

	const REPETITO_NAME = 'Znovupoužiteľný obal REPETITO';
	const REPETITO_DESC = 'Pomôžte nám znížiť množstvo odpadu z online nákupov.';
	const REPETITO_LONG = 'Čo je znovupoužiteľný obal?';
	const REPETITO_TEXT = 'Doručiť moju objednávku v znovupoužiteľnom obale';
	const REPETITO_LINK = 'https://www.repetito.sk/informacie-o-balikoch/';

	const PACKAGES = array(
		array(
			'name'     => 'soft_s',
			'volume'   => 8750,
			'niceName' => 'Soft S',
			'width'    => 35,
			'height'   => 25,
			'length'   => 10,
		),
		array(
			'name'     => 'soft_m',
			'volume'   => 28000,
			'niceName' => 'Soft M',
			'width'    => 50,
			'height'   => 35,
			'length'   => 16,

		),
		array(
			'name'     => 'soft_l',
			'volume'   => 46200,
			'niceName' => 'Soft L',
			'width'    => 60,
			'height'   => 35,
			'length'   => 22,
		),
		array(
			'name'     => 'box_s',
			'volume'   => 2079,
			'niceName' => 'Box S',
			'width'    => 21,
			'height'   => 11,
			'length'   => 9,
		),
		array(
			'name'     => 'box_m',
			'volume'   => 6468,
			'niceName' => 'Box M',
			'width'    => 28,
			'height'   => 21,
			'length'   => 11,
		),
	);

	/**
	 * Renders repetito fee checkbox on front-end
	 * `render` methods
	 *
	 * @param array $tags An array of shortcode tags.
	 */
	protected function __construct() {
		add_action( 'woocommerce_review_order_before_payment', array( $this, 'showFeeCheckbox' ), 10 );
		add_action( 'woocommerce_before_calculate_totals', array( $this, 'maybeUnsetFee' ), 10 );

		add_filter( 'woocommerce_form_field', array( $this, 'formatChecbox' ), 10, 4 );

		add_action( 'wp_ajax_nopriv_apply_fee', array( $this, 'applyFee' ) );
		add_action( 'wp_ajax_apply_fee', array( $this, 'applyFee' ) );

		add_action( 'wp_ajax_nopriv_apply_reload', array( $this, 'reload' ) );
		add_action( 'wp_ajax_apply_reload', array( $this, 'reload' ) );

		add_action( 'woocommerce_cart_calculate_fees', array( $this, 'calculateFee' ), 10, 1 );
	}

	private function countCartVolume( array $items ): int {
		$cart_total = array();

		foreach ( $items as $item => $values ) {
			$_product = wc_get_product( $values['data']->get_id() );
			if ( $_product->get_length() == '' || $_product->get_width() == '' || $_product->get_height() == '' ) {
				return 0;
			}
			$prod_m3 = (int) $_product->get_length() * (int) $_product->get_width() * (int) $_product->get_height();

			$prod_m3 = $values['quantity'] * $prod_m3;
			array_push( $cart_total, $prod_m3 );
		}

		return array_sum( $cart_total );
	}

	private function getLongest( array $items, string $size ): int {
		$all_items = array();

		foreach ( $items as $item => $values ) {
			$_product = wc_get_product( $values['data']->get_id() );
			switch ( $size ) {
				case 'width':
					$measure = $_product->get_width();
					break;

				case 'height':
					$measure = $_product->get_height();
					break;

				case 'length':
					$measure = $_product->get_length();
					break;

				default:
					break;
			}

			array_push( $all_items, $measure );
		}

		return (int) max( $all_items );
	}


	private function feeEnabled(): bool {
		if ( ! class_exists( 'woocommerce' ) ) {
			return false;
		}

		if ( true === WC()->cart->is_empty() ) {
			return false;
		}

		if ( false === get_option( 'repetito-settings' ) ) {
			return false;
		}

		if ( false == $this->countryEnabled() ) {
			return false;
		}

		$cart_weight  = WC()->cart->get_cart_contents_weight();
		$fee_treshold = (int) get_option( 'repetito-fees' )['weight'];

		if ( $cart_weight > $fee_treshold ) {
			return false;
		}

		$cart_volume = $this->countCartVolume( WC()->cart->get_cart() );

		// get all packages
		$all_packages = array_map(
			function ( $p ) {
				$p['active'] = (bool) get_option( 'repetito-packages_' . $p['name'], false );
				return $p;
			},
			self::PACKAGES
		);

		// remove un-active packages
		$available_packages = array_filter(
			$all_packages,
			function ( $p ) {
				return $p['active'] === true;
			}
		);

		// sort packages by smallest so we can start with the smallest for economy reasons
		usort(
			$available_packages,
			function ( $a, $b ) {
				return ( $a['volume'] <=> $b['volume'] );
			}
		);

		$final_package = '';
		foreach ( $available_packages as $package ) {
			// if all the sizes fits, set packaging and dont continue the loop
			if ( BoxPacker::fitsIntoPackage( WC()->cart->get_cart(), $package, WC()->cart->get_cart_contents_count() ) === false ) {
				continue;
			} else {
				$final_package = $package['name'];
				break;
			}
		}

		// if no package found - abort
		if ( $final_package === '' ) {
			return false;
		}

		$repetitoActive = get_option( 'repetito-settings' )['serviceActive'] ?? false;

		if ( $repetitoActive === true ) {
			return true;
		}

		return false;
	}

	public function maybeUnsetFee(): void {
		if ( false === $this->feeEnabled() || false === $this->countryEnabled() ) {
			$fees = WC()->cart->get_fees();
			foreach ( $fees as $key => $fee ) {
				if ( $key === 'znovupouzitelny-obal-repetito' ) {
					unset( $fees[ $key ] );
				}
			}
			WC()->cart->fees_api()->set_fees( $fees );
			WC()->session->__unset( 'repetito_fee' );
		}
	}

	public function showFeeCheckbox(): void {
		echo do_shortcode( '[repetito]' );

		if ( false === $this->feeEnabled() || false === $this->countryEnabled() ) {
			return;
		}

		$fee_price = (float) get_option( 'repetito-fees' )['price'];
		woocommerce_form_field(
			'repetito_fee',
			array(
				'type'    => 'checkbox',
				'label'   => sprintf(
					__( '%1$s! (%2$s %3$s)', 'repetitio' ),
					__( self::REPETITO_TEXT, 'repetito' ),
					$fee_price,
					get_woocommerce_currency_symbol()
				),
				'default' => WC()->session->get( 'repetito_fee' ) ? true : false,
			),
			''
		);
	}

	public function applyFee() {
		if ( check_ajax_referer( NAME, 'nonce' ) ) {
			$fee = filter_var( $_POST['applyFee'], FILTER_SANITIZE_NUMBER_INT );
			WC()->session->set( 'repetito_fee', ( $fee ? '1' : '0' ) );

			wp_send_json_success();
		} else {
			wp_send_json_error();
		}
	}

	public function reload() {
		if ( check_ajax_referer( NAME, 'nonce' ) ) {
			wp_send_json_success( array( 'show' => ( (int) $this->countryEnabled() && (int) $this->feeEnabled() ) ) );
		} else {
			wp_send_json_error();
		}
	}

	private function countryEnabled() {
		$country         = WC()->customer->get_shipping_country() ?: WC()->customer->get_billing_country();
		$enabled_country = get_option( 'repetito-fees' )['country'];

		return $country == $enabled_country;
	}

	public function calculateFee( $cart ) {
		if ( false == $this->countryEnabled() ) {
			WC()->session->__unset( 'repetito_fee' );
		}
		if ( WC()->session->get( 'repetito_fee' ) ) {
			$fee_price = (float) get_option( 'repetito-fees' )['price'];
			$tax       = get_option( 'repetito-fees' )['tax'];

			$cart->add_fee( __( self::REPETITO_NAME, 'repetito' ), $fee_price, true, $tax );
		}
	}

	public function formatChecbox( $field, $key, $args, $value ) {
		if ( 'repetito_fee' === $key && is_checkout() ) {
			$field    = '<div id="repetito">' . $field . '</div>';
			$optional = '&nbsp;<span class="optional">(' . esc_html__( 'optional', 'woocommerce' ) . ')</span>';
			$field    = str_replace( $optional, '', $field );

			$settings = get_option( 'repetito-settings' );
			$shown    = $settings['showImage'] ?? false;

			if ( true == $shown ) {
				$image = plugins_url( 'repetito/assets/website/static/logo.png' );
			}

			$name      = __( self::REPETITO_NAME, 'repetito' );
			$desc      = __( self::REPETITO_DESC, 'repetito' );
			$desc_long = __( self::REPETITO_LONG, 'repetito' );

			if ( $image && '' !== $image ) {
				$field = str_replace( 'class="woocommerce-input-wrapper">', 'class="woocommerce-input-wrapper"><img src="' . $image . '" width="1896" height="531"><span class="inner">', $field );
			} else {
				$field = str_replace( 'class="woocommerce-input-wrapper">', 'class="woocommerce-input-wrapper"><span class="inner">', $field );
			}

			if ( $desc ) {
				$field = str_replace( 'class="inner">', 'class="inner"><span class="desc">' . esc_html( $desc ) . ' </span><a href="' . self::REPETITO_LINK . '" target="_blank">' . $desc_long . '</a>', $field );
			}

			if ( $name ) {
				$field = str_replace( 'class="inner">', 'class="inner"><span class="name">' . esc_html( $name ) . '</span>', $field );
			}

			$field = str_replace( '</label>', '</span></label>', $field );

			if ( WC()->session->get( 'repetito_fee' ) ) {
				$field = str_replace( 'id="repetito_fee"', 'id="repetito_fee" checked ', $field );
			}
		}
		return $field;
	}
}
